@@ -4,16 +4,16 @@ import shutil
 from base64 import b64decode
 from pathlib import Path
 from time import sleep
-from typing import Callable, Dict, Iterator, Tuple
+from typing import Callable, Iterator, Tuple
 from unittest.mock import PropertyMock, create_autospec
 from urllib.parse import urljoin
 
-import attr
 import encode_utils as eu
 import pytest
 import requests
 from encode_utils.connection import Connection
 from pytest_mock.plugin import MockFixture
+from WDL import parse_document
 
 import docker
 from accession import backends
@@ -68,7 +68,10 @@ def local_encoded_server(
     client = docker.from_env()
     try:
         container = client.containers.run(
-            encoded_docker_image, detach=True, ports={"8000/tcp": host_port}
+            encoded_docker_image,
+            detach=True,
+            ports={"8000/tcp": host_port},
+            auto_remove=True,
         )
     except Exception as e:
         raise RuntimeError(
@@ -175,6 +178,15 @@ def mirna_replicated_metadata_path() -> str:
     return str(metadata_json_path)
 
 
+@pytest.fixture
+def wdl_workflow():
+    workflow = (
+        'workflow myWorkflow { meta { version: "v1.2.3" } call myTask } task myTask {'
+        " command { bar }}"
+    )
+    return workflow
+
+
 @pytest.fixture
 def encode_file_no_qc():
     return EncodeFile({"@id": "/files/foo/", "quality_metrics": []})
@@ -203,7 +215,7 @@ def encode_document(encode_attachment, encode_common_metadata):
 
 
 @pytest.fixture
-def gsfile():
+def gsfile(mocker):
     task = {
         "inputs": {
             "prefix": "pooled-pr1_vs_pooled-pr2",
@@ -212,91 +224,55 @@ def gsfile():
         "outputs": {},
     }
     my_task = Task("my_task", task)
-    return GSFile(
-        "foo", "gs://abc/spam.fastq.gz", md5sum="123", size="456", task=my_task
+    mocker.patch(
+        "accession.file.GSFile.md5sum", mocker.PropertyMock(return_value="123")
     )
-
-
-@attr.s(auto_attribs=True)
-class MockFile:
-    filename: str
-    size: int
-    md5sum: str
+    return GSFile(key="foo", name="gs://abc/spam.fastq.gz", task=my_task, client="bar")
 
 
 @pytest.fixture
-def mock_file() -> MockFile:
-    return MockFile("gs://foo/bar", 123, "abc")
+def mock_file(mocker):
+    file = mocker.Mock(
+        filekeys=["foo"], filename="gs://foo/bar", size=123, md5sum="abc"
+    )
+    return file
 
 
-class MockGCBackend(backends.GCBackend):
-    def __init__(self, bucket: str):
-        self.client = MockGCClient()
-        self.bucket = self.client.get_bucket(bucket)
+@pytest.fixture
+def mock_gs_file(mocker):
+    # md5sum is yN0BGTic4eg+yn7K3BVlGw== in base 64
+    file = mocker.MagicMock(
+        filekeys=["foo"],
+        filename="gs://foo/bar",
+        size=3,
+        md5sum="c8dd0119389ce1e83eca7ecadc15651b",
+    )
+    file.blob.download_as_string.return_value = b'{"foobar": "bazqux"}'
+    return file
 
 
-class MockBucket:
-    def __init__(self, bucket: str):
-        self.name = bucket
+class LocalMockGsFile:
+    SCHEME = "gs://"
 
-    def list_blobs(self) -> None:
-        pass
+    def __init__(self, key, name, task=None, used_by_task=None, client=None):
+        self.filekeys = [key]
+        self.filename = name
+        self.task = task
+        self.size = 3
+        self.used_by_tasks = [used_by_task] if used_by_task else []
+        self._md5_lookup = None
 
-
-class MockGCClient:
-    def get_bucket(self, bucket: str) -> MockBucket:
-        return MockBucket(bucket)
-
-
-@attr.s(auto_attribs=True)
-class MockBlob:
-    """
-    Using a dataclass would work here too, but attrs works for Python 3.6 as well.
-    """
-
-    path: str
-    bucket: MockBucket
-    size: int = 3
+    @property
+    def md5_lookup(self):
+        if self._md5_lookup is None:
+            md5_file = Path(__file__).resolve().parent / "data" / "gcloud_md5s.json"
+            with open(md5_file) as f:
+                self._md5_lookup = json.load(f)
+        return self._md5_lookup
 
     @property
     def md5sum(self) -> str:
-        # yN0BGTic4eg+yn7K3BVlGw== in base 64
-        return "c8dd0119389ce1e83eca7ecadc15651b"
-
-    @property
-    def id(self) -> str:
-        return f"{self.bucket.name}/{self.path}"
-
-    @property
-    def public_url(self) -> str:
-        url = urljoin("https://www.mystorageapi.com", f"{self.bucket.name}/{self.path}")
-        return url
-
-    def reload(self) -> None:
-        pass
-
-    def download_as_string(self) -> bytes:
-        return b'{"foobar": "bazqux"}'
-
-
-def load_md5_map() -> Dict[str, str]:
-    md5_file = Path(__file__).resolve().parent / "data" / "gcloud_md5s.json"
-    with open(md5_file) as f:
-        md5_lookup = json.load(f)
-    return md5_lookup
-
-
-class LocalMockBlob(MockBlob):
-    def __init__(self, path: str, bucket: MockBucket, size: int = 3):
-        self.path = path
-        self.bucket = bucket
-        self.size = size
-
-    md5_lookup = load_md5_map()
-
-    @property
-    def md5sum(self) -> str:
-        b64_md5 = self.md5_lookup[f"gs://{self.bucket.name}/{self.path}"]
+        b64_md5 = self.md5_lookup[self.filename]
         return b64decode(b64_md5).hex()
 
     def download_as_string(self) -> bytes:
@@ -304,43 +280,67 @@ class LocalMockBlob(MockBlob):
         We use the concatenation of the last two path elements (glob and filename) to
         find the file in the test data, which should be unique
         """
-        file_name = self.path.split("/")[-1]
+        file_name = self.filename.split("/")[-1]
         current_dir = Path(__file__).resolve()
         file_path = current_dir.parent / "data" / "files" / file_name
         with open(file_path, "rb") as f:
             binary_data = f.read()
         return binary_data
 
+    def read_json(self):
+        """
+        Read file and convert to JSON
+        """
+        return json.loads(self.download_as_string().decode())
+
+    def get_task(self) -> Task:
+        """
+        Either gets the `File`'s task or raises a ValueError.
+        """
+        task = self.task
+        if task is None:
+            raise ValueError("No task found")
+        return task
+
+    def get_uri_without_scheme(self) -> str:
+        return self.filename.split(self.SCHEME)[-1]
+
+    def get_filename_for_encode_alias(self) -> str:
+        return self.get_uri_without_scheme().replace("/", "-")
+
 
 @pytest.fixture
-def mock_gc_backend(mocker) -> MockGCBackend:
+def mock_gc_backend(mocker):
     """
-    Fully mocked fixture returning an instance of MockGCBackend that does not initialize
-    a Google Cloud client or make Google Cloud API calls.
+    A mocked instance of GCBackend that cannot initialize a Google Cloud client or make
+    Google Cloud API calls.
 
     mocker fixture can only be used with function-scoped pytest fixtures for now:
     https://github.com/pytest-dev/pytest-mock/issues/136
     """
-    mocker.patch.object(backends, "GcsBlob", MockBlob)
-    mock_gc_backend = MockGCBackend("accession-test-bucket")
+    mocker.patch("accession.file.GSFile", mock_gs_file)
+    mocker.patch("accession.backends.GCBackend.client", mocker.PropertyMock())
+    mock_gc_backend = backends.GCBackend()
     return mock_gc_backend
 
 
 @pytest.fixture
-def mock_accession_gc_backend(mocker) -> MockGCBackend:
+def mock_accession_gc_backend(mocker):
     """
     Similar to mock_gc_backend, except it provides real md5sums and can access real test
     data files.
     """
-    mocker.patch.object(backends, "GcsBlob", LocalMockBlob)
-    mock_gc_backend = MockGCBackend("encode-processing")
+    mocker.patch("accession.backends.GCBackend.client", mocker.PropertyMock())
+    mocker.patch("accession.file.GSFile", LocalMockGsFile)
+    mocker.patch("accession.backends.GSFile", LocalMockGsFile)
+    mock_gc_backend = backends.GCBackend()
     return mock_gc_backend
 
 
 @pytest.fixture
 def accessioner_factory(
     mocker: MockFixture,
-    mock_accession_gc_backend: MockGCBackend,
+    mock_accession_gc_backend,
     local_encoded_server: str,
     environ_api_credentials: None,
     lab: str,
@@ -379,13 +379,15 @@ def accessioner_factory(
     return _accessioner_factory
 
 
-class MockMetadata:
-    content = {"id": "foo", "workflowRoot": "gs://foo/bar", "calls": {}}
-    workflow_id = "123"
-
-
 @pytest.fixture
-def mock_metadata():
+def mock_metadata(wdl_workflow):
+    class MockMetadata:
+        content = {"id": "foo", "workflowRoot": "gs://foo/bar", "calls": {}}
+        workflow_id = "123"
+
+        def get_parsed_workflow(self):
+            return parse_document(wdl_workflow)
+
     return MockMetadata()
 
 
@@ -402,8 +404,8 @@ def mock_accession_steps():
 @pytest.fixture
 def mock_accession(
     mocker: MockFixture,
-    mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetadata,
+    mock_accession_gc_backend,
+    mock_metadata,
     mock_accession_steps: MockAccessionSteps,
     server_name: str,
     common_metadata: EncodeCommonMetadata,
@@ -473,8 +475,8 @@ def mock_accession_not_patched(
 @pytest.fixture
 def mock_accession_chip(
     mocker: MockFixture,
-    mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetadata,
+    mock_accession_gc_backend,
+    mock_metadata,
     mock_accession_steps: MockAccessionSteps,
     server_name: str,
     common_metadata: EncodeCommonMetadata,
@@ -519,11 +521,7 @@ def mock_accession_chip(
 
 @pytest.fixture
 def mock_accession_unreplicated(
-    mocker: MockFixture,
-    mock_accession_gc_backend: MockGCBackend,
-    mock_metadata: MockMetadata,
-    lab: str,
-    award: str,
+    mocker: MockFixture, mock_accession_gc_backend, mock_metadata, lab: str, award: str
 ) -> Accession:
     """
     Mocked accession instance with dummy __init__ that doesn't do anything and pre-baked
@@ -537,7 +535,9 @@ def mock_accession_unreplicated(
                 {
                     "@id": "foo",
                     "assay_term_name": "microRNA",
-                    "replicates": [{"biological_replicate_number": 1}],
+                    "replicates": [
+                        {"biological_replicate_number": 1, "status": "released"}
+                    ],
                 }
             )
         ),
